# 国际化i18n

Mpx支持国际化i18n，使用方式及支持能力与vue-i18n非常接近

## 使用方法

Mpx自带i18n能力，无需额外安装插件。由于小程序模板中的i18n函数是通过wxs编译注入进行实现，我们需要将i18n配置传入到MpxWebpackPlugin中来使i18n生效，这是与vue-i18n最大的区别

### 配置方法

i18n配置传入到MpxWebpackPlugin选项中即可生效，额外支持messagesPath/dateTimeFormatsPath/numberFormatsPath配置，其余配置参考vue-i18n

```js
new MpxWebpackPlugin({
  i18n: {
    locale: 'en-US',
    // messages既可以通过对象字面量传入，也可以通过messagesPath指定一个js模块路径，在该模块中定义配置并导出，dateTimeFormats/dateTimeFormatsPath和numberFormats/numberFormatsPath同理
    messages: {
      'en-US': {
        message: {
          hello: '{msg} world'
        }
      },
      'zh-CN': {
        message: {
          hello: '{msg} 世界'
        }
      }
    },
    // messagesPath: path.resolve(__dirname, '../src/i18n.js')
  }
})
```

### 在js中使用

同vue-i18n，在组件中直接调用翻译函数使用，目前js中支持了vue-i18n中全部的翻译函数，包括$t/$tc/$te/$d/$n，详细使用方法参考vue-i18n

```js
createComponent({
  ready () {
    console.log(this.$t('message.hello', { msg: 'hello' }))
  },
  computed: {
    formatedDatetime () {
      return this.$d(new Date(), 'long')
    }
  }
})
```

### 在模板中使用

类似于vue-i18n，在模板的Mustache插值中直接调用翻译函数，目前由于wxs执行环境的限制，模板上可直接使用的翻译函数包括$t/$tc/$te，如果需要格式化数字或日期可以使用对应的js翻译函数和Mpx提供的计算属性结合实现

```html
<view>
  <view>{{ $t('message.hello', { msg: 'hello' }) }}</view>
  <!-- formatedDatetime计算属性定义见上例，可基于locale变更响应刷新 -->
  <view>{{formatedDatetime}}</view>
</view>
```

### 动态变更locale
类似于vue-i18n，可实现全局local变更及局部local变更，并自动更新视图，使用方式如下
```js
import mpx, { createComponent } from '@mpxjs/core'

createComponent({
  ready () {
    // 局部locale变更，生效范围为当前组件内
    this.$i18n.locale = 'en-US'
    setTimeout(() => {
      // 全局locale变更，生效范围为项目全局
      mpx.i18n.locale = 'ja-JP'
    }, 1000)
  }
})
```

### 平台支持

由于Mpx中i18n实现依赖类wxs能力，目前支持除头条之外的所有小程序平台(微信/支付宝/qq/百度)，在输出web时，构建会自动引入vue-i18n并进行安装配置，无需修改任何代码即可按照预期正常工作。


